#ifndef _CommandLine_h_
#define _CommandLine_h_

namespace ge
{

    struct CommandLineItem
    {
        string key; // without "--"
        string val;  // can be empty
    };

    class CommandLine
    {
    protected:
        string program;
        string command;

        map <string, CommandLineItem> mapItems;
        vector <string> vMustHaveKeys;
        vector <string> vOptionalKeys;

        void fromAdTest(); // called internally within tests

        string ocdModuleName;

        string getKey(const string & key) const;

    public:


        CommandLine(int argc=0, char ** argv=NULL);


        void fromOCD();
        void fromMain(int argc, char * argv[]);

        void addMustHaveKeys(const string & keysLine); // string which contains keys separated by ',' like "width,height,length"
        void addOptionalKeys(const string & keyLine); // string which contains keys separated by ',' like "width,height,length"

        void setArgument(const string & key, const string &arg="");
        void setChar(const string & key, const char val);
        void setShort(const string & key, const short val);
        void setInt(const string & key, const int val);
        void setLONG(const string & key, const int64_t val);
        void setDouble(const string & key, const double val);
        void setFloat(const string & key, const float val);
        void setBool(const string & key, const bool flag);

        string getFirstMissingKey() const; // if returns "" everything is allright
        string buildCommandLineExample(bool flagIncludeOptionalKeys=false) const;
        bool checkKeys() const; // returns true if all keys are found
        int showExample(const string &strExample);

        bool hasItem(const string & key) const;
        string getArgument(const string & key) const;
        void show(int space=0) const;

        // interpret arguments as ..... simple types with default argument, if key not available
        bool isBool(const string & key, bool defaultVal=false)const;
        bool getBool(const string & key, bool defaultVal=false)const;

        // method to access command line arguments // "--size 100" for example
        int getInt(const string & key, int defaultVal=0)const;
        unsigned int getUInt(const string & key, unsigned int defaultVal=0)const;
        int64_t getLONG(const string & key, int64_t defaultVal=0)const;
        uint64_t getULONG(const string & key,uint64_t defaultVal=0)const;
        float getFloat(const string & key, float defaultVal=0)const;
        double getDouble(const string & key, double defaultVal=0)const;
        string getString(const string & key, string defaultVal="")const;


        string getCommand() const;


    };



    typedef int (*POINTER_TEST_COMMANDLINE)(CommandLine & cmd);


    struct TestContext
    {
        POINTER_TEST_COMMANDLINE pointerTest = 0;

        TestContext();
        virtual ~TestContext();
        TestContext(POINTER_TEST_COMMANDLINE pointerTest);

        int execTest(CommandLine & cmd);

        bool isEmpty() const;

        static TestContext empty;
    };


    struct TestContextMap
    {
        map<string , TestContext > mapTC;


        TestContextMap();
        virtual ~TestContextMap();

        bool hasTest(const string & key);
        void addTestContext(const string & key, TestContext tc);

    };


    struct TestContextAdder
    {
        TestContextAdder(POINTER_TEST_COMMANDLINE pointerTest, const string & key, const string & description="");
        static TestContext & getTestContext(const string & key);
        static TestContextMap * tcm;
    };


}



#endif
